using UnityEngine;
using System.Collections;

[ RequireComponent(typeof(CharacterController)) ]
[ RequireComponent(typeof(HomingDevice)) ]

public class OxygenPlayerChar : MonoBehaviour 
{
	
	public float oxygenLevelMax = 100;
	public float oxygenUseIdle = 0.01f;
	public float oxygenUseMoving = 0.01f;
	public float oxygenUseJump;
	public float [] oxygenDangerLevels;
	public bool deathOnOxygenOut = true;
	
	public KeyCode jumpKey;
	
	public AudioSource runBreatheSound;
	public AudioSource standBreatheSound;
	public AudioSource struggleBreatheSound;
	public AudioSource oxygenWarningSound;
	public AudioSource [] jumpSounds;
	public AudioSource [] landSounds;
	
	public TextMesh oxygenLevelDisplay;
	public GameObject oxygenWarningDisplay;
	
	private float oxygenWarningClock = 0;
	private float oxygenLevel;
	private AudioSource internalBreathingSound;
	private bool [] oxygenDangerSoundSwitches;
	private bool onGround = true;
	
	//UNITY CALLBACKS
	// Use this for initialization
	void Start () 
	{
		oxygenDangerSoundSwitches = new bool[oxygenDangerLevels.Length];
		
		for(int i = 0; i < oxygenDangerSoundSwitches.Length; i++)
		{
			oxygenDangerSoundSwitches[i] = false;
		}
		
		internalBreathingSound = standBreatheSound;
		oxygenLevel = oxygenLevelMax;
		
		Screen.showCursor = false;
	}
	
	// Update is called once per frame
	void Update () 
	{
		if(Input.GetAxis("Horizontal") == 0 && Input.GetAxis ("Vertical") == 0)
		{
			
			oxygenLevel -= oxygenUseIdle*Time.deltaTime;
			
			if(internalBreathingSound != standBreatheSound)
			{
				internalBreathingSound.Stop ();
				internalBreathingSound = standBreatheSound;
				
			}
			
		}
		else
		{
			oxygenLevel -= oxygenUseMoving*Time.deltaTime;
			
			if(internalBreathingSound != runBreatheSound)
			{	
				internalBreathingSound.Stop ();
				internalBreathingSound = runBreatheSound;
			}
		}
		
		if(Input.GetKeyDown(jumpKey))
		{
			oxygenLevel -= oxygenUseJump;
			
			if(onGround)
			{
				PlayRandomSound(ref jumpSounds);
				onGround = false;
			}
		}
		
		for(int i = 0; i < oxygenDangerLevels.Length; i++)
		{
			if(oxygenLevel <= oxygenDangerLevels[i])
			{
				if(!oxygenDangerSoundSwitches[i])
				{
					if(oxygenWarningSound != null)
					{
						oxygenWarningSound.Play ();
					}
					oxygenDangerSoundSwitches[i] = true;
					oxygenWarningClock = 3;
				}
			}
		}
		
		if(oxygenLevel < 0)
		{
			oxygenLevel = 0;
			
			if(deathOnOxygenOut)
			{
				Destroy (gameObject);
			}
			
		}
		
		if(oxygenLevelDisplay != null)
		{
			oxygenLevelDisplay.text = oxygenLevel.ToString ("#00.0");
		}
		
		if(internalBreathingSound != null)
		{
			if(!internalBreathingSound.isPlaying)
			{
				internalBreathingSound.Play();
			}
		}
		
		if(oxygenWarningDisplay != null)
		{
			if(oxygenWarningClock > 0)
			{
				oxygenWarningDisplay.SetActive(true);
				ActivateChildren(oxygenWarningDisplay.transform);
				oxygenWarningClock -= Time.deltaTime;
			}
			else
			{
				DeactivateChildren(oxygenWarningDisplay.transform);
				ActivateChildren(oxygenWarningDisplay.transform);
				oxygenWarningDisplay.SetActive(false);
				
			}
		}
	}
	
	void OnControllerColliderHit(ControllerColliderHit hit) 
	{
		if(!onGround)
		{
			PlayRandomSound(ref landSounds);
		}
		onGround = true;
		
		if(hit.gameObject.tag == "person")
		{
			Physics.IgnoreCollision(collider,hit.collider);
		}
	}
	
	//UNITY CALLBACKS
	
	public float GetOxygenLevels()
	{
		if(oxygenLevel > 0)
		{	
			return oxygenLevel;
		}
		else
		{
			return 0;
		}
	}
	
	public float OxygenGive(float oxLevel)
	{
		if(oxygenLevel - oxLevel > 0)
		{
			oxygenLevel -= oxLevel;
			return oxLevel;
		}
		else if (oxygenLevel > 0)
		{
			oxygenLevel = 0;
			return oxygenLevel;
		}
		else
		{
			return 0;
		}
	}
	
	public void OxygenTake(float oxLevel)
	{
		oxygenLevel += oxLevel;
	}
	
	public void ActivateChildren(Transform root)
	{
		foreach(Transform c in root)
		{
			c.gameObject.SetActive(true);
			ActivateChildren (c);
		}
	}
	
	public void DeactivateChildren(Transform root)
	{
		foreach(Transform c in root)
		{
			c.gameObject.SetActive(false);
			ActivateChildren (c);
		}
	}
	
	public void PlayRandomSound(ref AudioSource [] sounds)
	{
		if(sounds.Length > 0)
		{
			int randDex = Random.Range (0,sounds.Length);
			sounds[randDex].Play();
		}		
		
	}
}
